#!/usr/local/bin/perl -w

=head1 SYNOPSIS

wp-to-p.pl infile >outfile

=head1 AUTHOR

Copyright (c) 2003 Scott Haywood.  All rights reserved.

This package is free software and is provided "as is" without express
or implied warranty.  It may be used, redistributed and/or modified
under the terms of the Perl Artistic License (see
http://www.perl.com/perl/misc/Artistic.html)

=head1 DESCRIPTION

Script to convert waypoints from the format used by GPSman, 
to a different format (see below).

=head1 SAMPLE INPUT

!Format: DDD 1 WGS 84
!Creation: no

% Note: waypoint fields are separated by tabs
!W:
S1	(comment)	29.664448500	-177.5772994403
!NB: This is an extended,
multi-line comment.

S2	(comment)	29.666599631	-177.5788819436
S3	(comment)	29.665762782	-177.5725090150
S4	(comment)	29.667195082	-177.5688075665
S5	(comment)	29.663922787	-177.5694995765
S6	(comment)	29.663418531	-177.5733565930
S7	(comment)	29.661814570	-177.5789999608

=head1 SAMPLE OUTPUT

Datum,WGS84,WGS84,0,0,0,0,0
WP,D,S1 , 29.664448500, -177.5772994403,12/31/1989,00:00:00,11-MAY-02 12:13 
WP,D,S2 , 29.666599631, -177.5788819436,12/31/1989,00:00:00,11-MAY-02 12:38 
WP,D,S3 , 29.665762782, -177.5725090150,12/31/1989,00:00:00,11-MAY-02 13:05 
WP,D,S4 , 29.667195082, -177.5688075665,12/31/1989,00:00:00,11-MAY-02 13:27 
WP,D,S5 , 29.663922787, -177.5694995765,12/31/1989,00:00:00,11-MAY-02 13:47 
WP,D,S6 , 29.663418531, -177.5733565930,12/31/1989,00:00:00,11-MAY-02 14:10 
WP,D,S7 , 29.661814570, -177.5789999608,12/31/1989,00:00:00,11-MAY-02 15:21 

=cut

$verbose = 1;
sub say {
	print STDERR @_ if $verbose;
}

sub get_wp {
	my $line = $_[0];
	chomp $line;
	$line =~ 
	/([^\t]+)   	#waypoint name
	\t
	([^\t]*)	#comment
	\t
	([^\t]+)	#latitude
	\t
	([^\t]+)	#longitude
	/x
		or die "bad waypoint: $_";
	my $name = $1;
	my $comment = $2;
	my $lat = $3;
	my $long = $4;
	say "< ", join(":", ($name, $comment, $lat, $long)), "\n";
	return { name=>$name, comment=>$comment, lat=>$lat, long=>$long };
}

############################################################
# process command-line
#
#@argv = @ARGV;

while (@ARGV && $ARGV[0] =~ /^-/) {
	$_ = shift;
	last if /^--$/;
	# ...
}

@wp = ();
%seen = ();
LINE: while (<>) {
	#print STDERR "LINE: ", $_;
	next LINE unless /^!W/;
	$_ = <>; ## read first waypoint line
	WPLINE: for (;;) {
		# check for end of file
		last LINE if ! defined($_);

		# check for end of this group of waypoints
		last WPLINE if /^\s*$/;  # it really won't have a space char

		# create a record
		my $wp = &get_wp( $_ );

		# read the extended comment, if any
		$_ = <>; ## read another line
		if (defined($_) && /^!NB:(.*)/) {
			#$_ = $1;
			SKIP_NB: for (;;) {
				$_ = <>; ## read another line
				last SKIP_NB if /^$/;  # empty line (no space char)
				last LINE if ! defined($_);
			}
			$_ = <>; ## read next waypoint line
		}

		# Add to our list of waypoints
		if (!defined($seen{$$wp{name}})) {
			push @wp, $wp;
			$seen{$$wp{name}} = 1;
		} else {
			print STDERR "duplicate: $$wp{name}\n";
		}
	}
}

if (@wp) {
	print <<'END' ;
Datum,WGS84,WGS84,0,0,0,0,0
END
}

sub olat {
	my $inlat = $_[0];
	if ($inlat =~ /^N(.*)/) { return $1; }
	elsif ($inlat =~ /^S(.*)/) { return '-'.$1; }
	else { return $inlat; }
}

sub olong {
	my $inlong = $_[0];
	if ($inlong =~ /^E(.*)/) { return $1; }
	elsif ($inlong =~ /^W(.*)/) { return '-'.$1; }
	else { return $inlong; }
}

$ltime = localtime; # e.g. "Tue Apr 22 10:02:57 2003"
my ($dayname, $m, $d, $t, $y) = split /\s+/, $ltime;
$y >= 2000 and $y = substr($y, 2);
$m = uc $m;
$t = substr($t,0,5);

# output lines similar to this:
#WP,D,S6 , 29.66341, -99.57335, ...(other junk)...

@sorted_wp = sort { $$a{name} cmp $$b{name} } @wp;
for my $wp (@sorted_wp) {
	my $name = uc $$wp{name};
	print "WP,D,$name , ";
	print &olat( $$wp{lat} ), ", ";
	print &olong( $$wp{long} ), ",";
	print "12/31/1989,00:00:00,";
	print "$d-$m-$y $t";
	print "\n";
}

