#!/usr/local/bin/perl -w

=head1 DESCRIPTION

Converts point count data into a tab-separated table
organized into rows by species, and in columns by point.

=head1 AUTHOR

Copyright (c) 2003 Scott Haywood (scotthaywood.com).  All rights reserved.

This package is free software and is provided "as is" without express
or implied warranty.  It may be used, redistributed and/or modified
under the terms of the Perl Artistic License (see
http://www.perl.com/perl/misc/Artistic.html)

=head1 SYNOPSIS

 pt-count.pl [options] [-p waypointfile] [-l listfile] infile

 Options:
 -a    Include all species.
       Create empty rows for species not recorded in infile.

 -l file   Read species symbols and names from file.
           Defaults to "species.txt".
 
 -O sym  Output a column for species encountered other than
         at the survey points.  The column will contain the
         symbol sym for each such species.

 -n    Use names.
       Output descriptive names of species, instead of symbols.

 -p file  Read coordinates of survey points from file.

 -r N   Treat the coordinates as decimal numbers,
        and round to N places.  Otherwise, output the
		coordinates unchanged.

 -u   Append unrecognized symbols.
      For symbols not in the species list file,
      treat them as additional species at the end of the list.

 -w   Include an additional time/weather summary.

 -z   Use zero to represent "not recorded".
      Output "0" instead of an empty string,
      for points where a species was not encountered

=head1 SAMPLE INPUT

 The list file:
 --
 amke American Kestral
 noca Northern Cardinal
 cach Carolina Chickadee
 bcti Black-crested Titmouse
 --

 The waypoint file:
 The names of the waypoints (the third field)
 must end with numbers, and must be be in
 numerical order, starting with 1.
 --
 WP,D,p1 , 30.215629, -97.650457
 WP,D,p2 , 30.212995, -97.645430
 --

 The point count data file:
 Lines beginning with '#' are comments, and lines
 beginning with '$' separate points.
 If the first line for a point begins with a
 number, it is assumed to be the time; the
 remainder of the line may contain weather info
 in the order shown (with any trailing part
 omitted).
 --
 # time temp[f] wind clouds[%]
 715 67 0 100
 noca
 cach 2
 $
 732 67f s/5 100%
 noca 5
 bcti 2
 amke
 --

=head1 SAMPLE OUTPUT

 The output fields are tab-separated.
 Lines are sorted by the order in the species list file.
 --
 lat:	30.215629	30.212995
 long:	-97.650457	-97.645430
 time	715	732
 temp	67	67
 wind	none	S/5
 clouds	100	100

 American Kestral		1
 Northern Cardinal	1	5
 Carolina Chickadee	2
 Black-crested Titmouse		2

 totals:	3	8
 --

=cut

$verbose = 1;
sub say {
	print STDERR @_ if $verbose;
}

#################################################

use Getopt::Std;
use species_list;
use p_waypoints;

# process command-line
#
$listfile = "species.txt";
$use_name = 0;
$use_zero = 0;
$show_all = 0;
$append = 0;
$sum_weather = 0;
$other_column = 0;
$other_sym = 'X';
$coord_places = 0; # zero means don't round
%opts = ();
getopts('uO:wanzl:p:r:', \%opts);
if (exists $opts{u}) {
	$append = 1;
}
if (exists $opts{O}) {
	$other_column = 1;
	$other_sym = $opts{O};
}
if (exists $opts{w}) {
	$sum_weather = 1;
}
if (exists $opts{a}) {
	$show_all = 1;
}
if (exists $opts{n}) {
		$use_name = 1;
}
if (exists $opts{z}) {
		$use_zero = 1;
}
if (exists $opts{l}) {
		$listfile = $opts{l};
}
if (exists $opts{p}) {
		$wpfile = $opts{p};
}
if (exists $opts{r}) {
		$coord_places = int($opts{r});
}

$timepat = qr!\d\d?:?\d\d!;
$point = 0;
$lines_this_point = 0;
$splist = species_list->new($listfile);
$splist->appending(1) if $append;
$got_data = 0;
$got_time = 0;
$others = 0;
LINE: while (<>) {
	#print STDERR "LINE: ", $_;
	if (/^\$/) { # points separated by "$" at start of line
		$others = 0;
		if (/^\$others/) { # other species seen; must be at end
			$others = 1;
		}
		elsif ($got_data) {
			$point++;
			$got_time = 0;
			$lines_this_point = 0;
		}
		next LINE;
	}
	if (/^\s*#/) { next LINE };
	@words = split;

	## non-point data
	##
	if ($others) {
		while (@words) {
			my $sym = shift @words;
			my $num = 1;
			last if $sym =~ /^#/;
			if (@words && $words[0] =~ /^\d/) {
				$num = shift @words;
			}
			$splist->add_nonpoint($sym,$num);
		}
		next LINE;
	}

	## point data
	##
	$lines_this_point++;
	if (@words && $words[0] =~ $timepat) {
		die "already got time, before $_\n" if $got_time;
		($time,$temp,$wind,$clouds) = (@words,'','','');
		$time =~ s/://;
		$temp =~ s/(.*)f$/$1/i;
		$wind =~ s/^0+$/none/ or $wind =~ /var/ or $wind = uc $wind;
		$splist->set_data('time',$point,$time);
		$splist->set_data('temp',$point,$temp);
		$splist->set_data('wind',$point,$wind);
		if (defined($clouds)) {
			$clouds =~ s/%$//;
			$splist->set_data('clouds',$point,$clouds)
		}
		$got_time = $got_data = 1;
	} else {
		while (@words) {
			my $sym = shift @words;
			my $num = 1;
			last if $sym =~ /^#/;
			if (@words && $words[0] =~ /^\d/) {
				$num = shift @words;
			}
			$splist->set_data($sym,$point,$num);
			$got_data = 1;
		}
	}
}

# Write the lat/long for each point (points in columns).
#
# The points are checked to insure correct order
#
$num_points = $splist->num_points;
if (defined($wpfile)) {
	$wp = p_waypoints->new($wpfile);
	@list = $wp->list;

	# The waypoints must be in numerical order.
	#
	for (my $i=1; $i <= $num_points; $i++) {
		die "no point $i in $wpfile" if ! defined($list[$i-1]);
		my $p = $list[$i-1];
		my ($num) = $$p{name} =~ /(\d+)$/;
		$num = -1 if ! defined($num);
		#print STDERR "i=$i, num = $num\n";
		die "In $wpfile, got $$p{name}, expected point $i\n"
			if $num != $i;
	}

	@syms = qw(lat long);
	my $format = '%s';
	$format = '%.' . $coord_places . 'f' if $coord_places;
	for my $sym (@syms) {
		if ($other_column) {
			print "\t";
		}
		print "$sym:";
		for my $p (@list) {
			printf("\t" . $format, $$p{$sym});
		}
		print "\n";
	}
}

# get symbols not in the original list
#
@added = $splist->appended;
my %added = map { $_ => 1 } @added;

# Get the list of species encountered
# outside of the point intervals
#
@np = $splist->get_nonpoint;
my %others = map { $_ => 1 } @np;

# write the weather and species data (one species per line)
#
@list = $splist->list;
%seen = ();
$zval = $use_zero ? 0 : '';
for $sp (@list) {
	my $sym = $$sp{sym};
	if ($sym eq 'blank') {
		print "\n";
		next;
	}
	@data = @{ $$sp{data} };
	next if (! $show_all && ! @data);
	$seen{$sym} = (@data > 0);

	if ($other_column) {
		# non-null if the species was encountered, but
		# not seen/heard during the point intervals.
		if ($others{$sym} && !$seen{$sym}) {
			print $other_sym;
		}
		print "\t";
	}

	print "A " if $added{$sym};
	print $use_name ? $$sp{name} : $sym;
	print "\t";
	print join("\t",map {defined($_)?$_:$zval} @data);
	print "\n";
}

# write the totals for the points
#
@totals = $splist->get_point_totals;
if (@totals) {
	print "\n";
	print "\t" if ($other_column);
	print "totals:\t";
	print join("\t",@totals), "\n";
}

# write the other species encountered
#
@np = $splist->get_nonpoint;
print "\nothers:\n" if (@np);
while (@np) {
	my ($sym, $val) = (shift @np, shift @np);
	next if $seen{$sym};
	print $use_name ? $splist->get_name($sym) : $sym;
	print "\t";
	print $val if $val > 1;
	print "\n";
};

# write a time/weather summary if requested
#
if ($sum_weather) {
	print "\n";
	@time = $splist->get_data('time');
	@temp = $splist->get_data('temp');
	@wind = $splist->get_data('wind');
	@clouds = $splist->get_data('clouds');
	#print STDERR "#time: ", 0+@time, "\n";
	#print STDERR "#temp: ", 0+@temp, "\n";
	#print STDERR "#wind: ",0+@wind, "\n";
	#print STDERR "#clouds: ",0+@clouds, "\n";
	my $i;
	for ($i=0; $i<@time; $i++) {
		print $i + 1, "\t";
		my $w = $wind[$i];
		$w =~ s/([\d-]+)/$1 MPH/;

		printf("Time: %04d.  Temp: %s F, Wind: %s, Sky: %s",
			$time[$i], $temp[$i], $w, $clouds[$i]);
		print "% clouds" if $clouds[$i] =~ /^\d+$/;
		print "\n";
	}
}

