
=head1 DESCRIPTION

Perl module to manage point-count data.
It maintains an array of species.
Each species has an array of point-count data.

=head1 AUTHOR

Copyright (c) 2003 Scott Haywood.  All rights reserved.

This package is free software and is provided "as is" without express
or implied warranty.  It may be used, redistributed and/or modified
under the terms of the Perl Artistic License (see
http://www.perl.com/perl/misc/Artistic.html)

=cut

package species_list;
use MyAutoLoader2;
use strict;

use vars qw(@ISA);
@ISA = qw(MyAutoLoader2);

# create a new list
sub new {
	my $that  = shift; # class name or object reference
	my $class = ref($that) || $that;

	my $filename = shift or die "no file name";

	my $input;
	unless (open($input, $filename)) {
	   print STDERR "Can't open $filename: $!\n";
	   return;
	}

	my %permitted = (filename=>$filename,
	                 list=>[],
					 hash=>{},
					 appending=>0);
	my $x = $class->SUPER::new(\%permitted, @_);

	my @newlist = ();
	my %newhash = ();
	$x->list(\@newlist);
	$x->hash(\%newhash);
	my @attr = ();
	while (<$input>) {
		if ( /^\s*\$attr\s+/ ) {
			@attr = split;
			for (@attr) {
				die "bad attr $_" if /sym|name|data/;
			}
			shift @attr;
		}
		elsif (! /^\w/) {
			next;
		}
		elsif (/(\w+)\s+(.*)/) {
			my ($sym,$name) = ($1,$2);
			chomp $name;
			&_append(\@newlist, \%newhash, $sym, $name, @attr);
		} else {
			die "bad format: $_";
		}
	}

	return $x;
}

# Looks up a species by symbol.
# Returns the hash structure associated with the symbol
#
sub get_species {
	my $self = shift;
	die "not an object" if ! ref($self);
	
	my $sym = shift or die "no sym";
	my $hash = $self->hash;
	my $index = $$hash{$sym};
	return undef if !defined($index);
	my @list = $self->list;
	return $list[$index];
}

# Looks up a species by symbol.
# Returns the descriptive name.
#
sub get_name {
	my $self = shift;
	die "not an object" if ! ref($self);
	
	my $sym = shift or die "no sym";
	my $species = $self->get_species($sym);
	return undef if !defined($species);
	return $$species{name};
}

# Looks up a species by symbol.
# Returns the point data.
#
sub get_data {
	my $self = shift;
	die "not an object" if ! ref($self);
	
	my $sym = shift or die "no sym";
	my $species = $self->get_species($sym);
	return undef if !defined($species);
	return @{ $$species{data} };
}
	
# Adds a record
#
sub _append {
	my $list = shift;
	my $hash = shift;
	my $sym = shift;
	my $name = shift;
	$$hash{$sym} = scalar @$list;
	my $obj = {sym=>$sym, name=>$name, data=>[], @_};
	push @$list, $obj;
	return $obj;
}
sub append {
	my $self = shift;
	die "not an object" if ! ref($self);
	
	my $sym = shift or die "no sym";
	my $name = shift or die "no name";
	my $attr = \@_;

	my $list = $self->getfield('list');
	my $hash = $self->hash;
	return &_append($list,$hash,$sym,$name,appended=>1,@$attr);
}

# Record the value of an item at an index,
# e.g. the number of a species at one point of a survey
#
sub set_data {
	my $self = shift;
	die "not an object" if ! ref($self);
	
	my $sym = shift or die "no sym";
	my $index = shift; die "no index" if !defined($index);
	my $value = shift; die "no value" if !defined($value);

	my $species = $self->get_species($sym);
	if (!defined($species)) {
		die "unknown symbol $sym" if !$self->appending;
		print STDERR 
		 "appending symbol -- $sym --, with data at index $index\n";
		$species = $self->append($sym, $sym, summable=>1);
	}
	my $data = $$species{data};
	$$data[$index] = $value;
}
	
# Records an item, but not at any index,
# e.g. a species encountered but not at a survey point.
#
sub add_nonpoint {
	my $self = shift;
	die "not an object" if ! ref($self);
	
	my $sym = shift or die "no sym";
	my $value = (shift or 1);

	my $species = $self->get_species($sym);
	if (!defined($species)) {
		die "unknown symbol $sym" if !$self->appending;
		print STDERR 
		 "appending symbol -- $sym --, with non-point data\n";
		$species = $self->append($sym, $sym, summable=>1);
	}
	$$species{nonpoint} = $value;
}

# Returns a sorted array of items recorded with add_nonpoint()
# Format is (item-a, value-a, item-b, value-b, ...)
#
sub get_nonpoint {
	my $self = shift;
	die "not an object" if ! ref($self);

	my @rv = ();
	my @list = $self->list;
	for my $sp (@list) {
		if (defined($$sp{nonpoint})) {
			push @rv, $$sp{sym};
			push @rv, $$sp{nonpoint};
		}
	}
	return @rv;
}

# Returns an array containing the number of "summable"
# entities recorded for each point.
#
sub get_point_totals {
	my $self = shift;
	die "not an object" if ! ref($self);

	my @rv = ();
	my @list = $self->list;
	for my $sp (@list) {
		next if ! $$sp{summable};
		my $data = $$sp{data};
		for (my $i=0; $i<@$data; $i++) {
			if (defined($$data[$i])) {
				$rv[$i] = ! defined($rv[$i])
				          ? $$data[$i] : $rv[$i] + $$data[$i];
			}
		}
	}
	for (@rv) { $_ = 0 if !defined($_)}
	return @rv;
}

# Returns a list of symbols which were automatically appended
#
sub appended {
	my $self = shift;
	die "not an object" if ! ref($self);

	my @rv = ();
	my @list = $self->list;
	for my $sp (@list) {
		next if ! $$sp{appended};
		push @rv, $$sp{sym};
	}
	return @rv;
	
}

# Returns the number of survey points,
# which is the maximum length of the data arrays
#
sub num_points {
	my $self = shift;
	die "not an object" if ! ref($self);

	my @list = $self->list;
	my $max = 0;
	for my $sp (@list) {
		my $data = $$sp{data};
		$max = @$data if ($max < @$data);
	}
	return $max;
}


1;
__END__

